#include "docs.h"

/*
 * Extract the next word from the given document (file) fp,  and put it into the string 'word'.
 * This function leaves the file pointing to the next character after the word that was read.
 *
 * Returns
 *   EOF        If there are no alnum's from the specified position up to the EOF.
 *   ERROR      In case of trouble.
 *   TRUE       In normal cases.
 *   DOC_START  When the TAG character is encountered (when scanning
 *              a collection of docs)
 */
int get_word(FILE* fp, char* word) {

    long wstart;  /* the position of the first byte of the word */
    long wend;    /* same for the last byte. */

    if (check_file(fp, "textfile in get_word") == ERROR)
        return ERROR;

    char c;
    while (!isalpha(c = getc(fp)))  /* skip non-alpha's */
    {
        if (c == EOF) {
            return (EOF);
        }
        if (c == TAG) { /* start-of-document in an archive is encountered */
            /* This check is used only when building an index */
            /* on an archive (= collection of docs) */
            word[0] = EOS;
            return (DOC_START);
        }
    }

    wstart = ftell(fp) - 1;  /* fp points to the second character of the word */
    while (isalpha(c = getc(fp))) /* absorb the rest of the word */
        ;

    if (c != EOF)
        ungetc(c, fp);  /* push back the non-alnum character */
    /* In case of EOF, we do not need to call ungetc, because 'fp' points to the EOF character,
     * which is exactly the character after the word. */

    wend = ftell(fp) - 1;  /* fp points to the char. after the word */
    /* even if this character is the EOF char. */

    fseek(fp, wstart, 0);
    fread(word, sizeof(char), wend - wstart + 1, fp);
    word[wend - wstart + 1] = EOS;

#ifdef DEBUG
    printf("get_word: start=%ld, end=%ld\n", wstart, wend);
    printf("  word: >|%s|<\n", word);
#endif

    return TRUE;
}
