#include "postings.h"

/* Forward declarations */
int is_full(int count);

/*
 * This function is passed two values: a pointer to an offset in the postings file, and a offset
 * value referring to the textfile. The routines get the record at the postings file offset and
 * checks to see if there is enough space for another text offset. If there is empty offset space
 * available in the same record, the number of text offsets in the record is incremented and
 * re-written to the record and the new text offset is added to the end of the record.
 *
 * If the current postings list is full, all the pointers are read into memory, and the current
 * record is added to a freelist of that size record. A freelist of every size record is kept by
 * keeping the head pointers in the first record of the file.
 *
 * A new free record of the next largest size, if available, is retrieved from the appropriate
 * head pointer at the beginning of the file. If there is no block of the appropriate size, the
 * new record is set up at the end of the file.
 */
void update_postings_file(long* postings_offset, long new_doc_offset) {
    /* goto postings offset */
    fseek(fp_postings, *postings_offset, 0);
    /* read in the number of postings pointers */
    long num_doc_offsets;
    fread(&num_doc_offsets, sizeof(num_doc_offsets), 1, fp_postings);

    /* check if there is space available for another pointer */
    if (!is_full(num_doc_offsets)) {
        /* update number of pointers */
        num_doc_offsets++;
        fseek(fp_postings, *postings_offset, 0);

        fwrite(&num_doc_offsets, sizeof(num_doc_offsets), 1, fp_postings);
        /* read past all old text pointers */
        fseek(fp_postings, (long) sizeof(new_doc_offset) * (num_doc_offsets - 1), 1);
        /* write new text pointer */
        fwrite(&new_doc_offset, sizeof(new_doc_offset), 1, fp_postings);

    } else { /* we need to create a new postings record and free old */
        /* get space to hold list of doc pointers */
        long* doc_offset_list = (long*) malloc(sizeof(long) * num_doc_offsets);
        check_malloc(doc_offset_list, "Doc Offset List");
        /* save position in record */
        long location = ftell(fp_postings);
        /* read in list of doc pointers */
        fread(doc_offset_list, sizeof(long), num_doc_offsets, fp_postings);

        /* add the current text pointer record to one of the free lists */
        free_record(*postings_offset, num_doc_offsets);

        /* get new block of next larger size */
        num_doc_offsets++;

        *postings_offset = get_record(num_doc_offsets);

        /* goto the new block on the disk */
        fseek(fp_postings, *postings_offset, 0);

        /* write new list of pointers */
        fwrite(&num_doc_offsets, sizeof(num_doc_offsets), 1, fp_postings);
        fwrite(doc_offset_list, sizeof(long), num_doc_offsets - 1, fp_postings);
        fwrite(&new_doc_offset, sizeof(long), 1, fp_postings);
    }
}

/* Check if any postings list is full. */
int is_full(int count) {
    int i = 0;
    while (count >= LIST_SIZE[i]) {
        if (LIST_SIZE[i] == count)
            return TRUE;
        i++;
    }
    return FALSE;
}
